<?php
/*
* Class Miniatura v1.0
*
* created by Jakub Sedláček
* visit http://js.ownage.sk/ for more information
* 
*/


class Miniatura {

  private $im;
  private $platno;
  private $povolene_typy = array("jpg"=>"image/jpeg","png"=>"image/png","gif"=>"image/gif");
  private $defaultKvalita = array("jpg"=>90,"png"=>9);
  private $chyba = "";
  private $obrazok_info;
  private $efekty = array();
  private $typ; // image/jpeg, image/png...
  private $ulozDo = NULL;
  private $platnoReady = false;
  private $antiAlias = true;
  private $vodoznakIm = array(); //vodoznak - resource
  private $vodoznakInfo = array(); //pole: [0] = sirka; [1] = vyska
  private $vodoznakXY = array(); // poloha "x", a "y"
  public $vodoznakOdsadenie = 5;
  public $cache = 0; //864000 = 10 dni v sekundach
  public $orezanie = false;
  public $rotujVodoznak = true;
  public $sirka = 0;
  public $vyska = 0;
  public $otoc = 0;
  public $vystup;
  public $maxEfektov = 5;
  public $pozadie = "255,255,255,127";
  private $pozadiePlatna = array();
  public $kvalita = 90;
  
  

  
  public function __construct($cesta_k_suboru,$povol_url=false) {
    $this->obrazok = new StdClass;
    $this->obrazok->sirka = 0;
    $this->obrazok->vyska = 0;
    $this->obrazok->mime = "";
    
    if(!filter_var($cesta_k_suboru, FILTER_VALIDATE_URL)) {
      if(!file_exists($cesta_k_suboru)) {
        $this->nastavChybu("Subor sa nenasiel.");
        return;
      }
    } else {
      if(!$povol_url) {
        $this->nastavChybu("URL nie je povolene.");
        return;
      }
    }
    
    $this->nastavObrazokInfo($cesta_k_suboru);
    
    if(!in_array($this->obrazok->mime,$this->povolene_typy)) {
      $this->nastavChybu("Nepovoleny typ suboru.");
      return;
    }
    $this->nastavIm($cesta_k_suboru);
  }
  
  private function nastavObrazokInfo($cesta_k_suboru) {
    $this->obrazok_info = @getimagesize($cesta_k_suboru);
    $this->obrazok->sirka = $this->obrazok_info[0];
    $this->obrazok->vyska = $this->obrazok_info[1];
    $this->obrazok->mime = $this->obrazok_info['mime'];
    $this->obrazok->zlava = 0;
    $this->obrazok->zhora = 0;
  }
  
  public function __destruct() {
	
		if(is_resource($this->im)) imagedestroy($this->im);
		if(is_resource($this->platno)) imagedestroy($this->platno);
	
	}
  
  public function nastavOrezanie($hodnota) {
    $this->orezanie = (bool)$hodnota;
  }
  
  
  private function nastavChybu($chybova_hlaska,$kresli=false) {
    $this->chyba = $chybova_hlaska;
    if($kresli) $this->kresliChybu();
  }
  
  public function pridajEfekt($efekt,$efektData="") {
    $this->efekty[] = array("nazov"=>$efekt,"data"=>$efektData);
  }
  
  private function vykonajEfekt($efekt,$efektData="") {
    if(is_resource($this->platno)) {
      switch($efekt) {
        case "ciernobiely":
          imagefilter($this->platno,IMG_FILTER_GRAYSCALE);
        break;
        case "sepia":
          imagefilter($this->platno,IMG_FILTER_GRAYSCALE);
          imagefilter($this->platno,IMG_FILTER_COLORIZE,100,50,0);
        break;
        case "malba":
          imagefilter($this->platno,IMG_FILTER_CONTRAST,-1000);
          imagefilter($this->platno,IMG_FILTER_BRIGHTNESS,10);
          imagefilter($this->platno,IMG_FILTER_GRAYSCALE);
        break;
        case "negativ":
          imagefilter($this->platno,IMG_FILTER_NEGATE);
        break;
        case "emboss":
          imagefilter($this->platno,IMG_FILTER_EMBOSS);
        break;
        case "rozmazanie":
          imagefilter($this->platno,IMG_FILTER_GAUSSIAN_BLUR);
        break;
        case "zvyraznihrany":
          imagefilter($this->platno,IMG_FILTER_EDGEDETECT);
        break;
        case "kontrast":
          imagefilter($this->platno,IMG_FILTER_CONTRAST,intval($efektData));
        break;
        case "jas":
          imagefilter($this->platno,IMG_FILTER_BRIGHTNESS,intval($efektData));
        break;
        case "pixeluj":
          imagefilter($this->platno,IMG_FILTER_PIXELATE,intval($efektData));
        break;
        case "kolorizuj":
          $exp = explode(",",$efektData);
          if(isset($exp[0]) && isset($exp[1]) && isset($exp[2])) {
            imagefilter($this->platno,IMG_FILTER_COLORIZE,intval($exp[0]),intval($exp[1]),intval($exp[2]));
          }
        break;
      }
    }
  }
  
  private function vykonajVsetkyEfekty() {
    if(count($this->efekty)>0) {
      $i=0;
      foreach($this->efekty as $efekt) {
        $i++;
        if($i > $this->maxEfektov) break;
        $this->vykonajEfekt($efekt['nazov'],$efekt['data']);
      }
    }
  }
  
  public function pridajEfekty($fx) {
    $exp = explode("|",$fx);
    foreach($exp as $efekt) {
      $efektData = NULL;
      $exp = explode(":",$efekt);
      if(count($exp)>1) list($efekt, $efektData) = $exp;
      $this->pridajEfekt($efekt,$efektData);
    }
  }
  
  private function kontrolaUdajovRozmery() {
    $sirka = intval($this->sirka);
    $vyska = intval($this->vyska);
    if($vyska>0) {
      $this->vyska = $vyska;
    } else {
      $this->vyska = $this->obrazok_info[1];
    }
    if($sirka>0) {
      $this->sirka = $sirka;
    } else {
      $this->sirka = $this->obrazok_info[0];
    }
  }
  
  private function kontrolaUdajovKvalita() {
    $this->kvalita = intval($this->kvalita);
    
    switch($this->typ) {
      case "image/jpeg":
        if($this->kvalita < 0 || $this->kvalita > 100) $this->kvalita = $this->defaultKvalita['jpg'];
      break;
      case "image/png":
        if($this->kvalita < 0 || $this->kvalita > 9) $this->kvalita = $this->defaultKvalita['png'];
      break;
    }
    
  }
  
  private function kontrolaUdajov() {
    
    $this->typ = $this->obrazok_info['mime'];
    if(array_key_exists($this->vystup,$this->povolene_typy)) {
      $this->typ = $this->povolene_typy[$this->vystup];
    }

    $this->kontrolaUdajovRozmery();
    $this->kontrolaUdajovKvalita();
    
    $this->cache = abs((int)$this->cache);
    $this->orezanie = (bool)$this->orezanie;
    $this->otoc = intval($this->otoc);
    
    $exp_pozadie = explode(",",$this->pozadie);
    if(isset($exp_pozadie[0]) && isset($exp_pozadie[1]) && isset($exp_pozadie[2])) {
      $this->pozadiePlatna = array(
        "r"=>intval($exp_pozadie[0]),
        "g"=>intval($exp_pozadie[1]),
        "b"=>intval($exp_pozadie[2]),
        "a"=>(isset($exp_pozadie[3]) ? intval($exp_pozadie[3]) : 0)
      );
    }
    
    
    $this->maxEfektov = abs(intval($this->maxEfektov));
    
    //vodoznak
    $this->rotujVodoznak = (bool)$this->rotujVodoznak;
    $this->vodoznakOdsadenie = abs(intval($this->vodoznakOdsadenie));
    
  }
  
  private function nastavRozmery() {
    $this->kontrolaUdajov();
    $sirka_zdroja = $this->obrazok->sirka;
    $vyska_zdroja = $this->obrazok->vyska;
    if($sirka_zdroja!=0 && $vyska_zdroja!="") {
      if($this->orezanie) {
        if(($this->sirka/$this->vyska) < ($this->obrazok->sirka/$this->obrazok->vyska)) {
          $this->obrazok->sirka = ($vyska_zdroja/$this->vyska)*$this->sirka;
          $this->obrazok->zlava = ($sirka_zdroja - $this->obrazok->sirka)/2;
        } else {
          $this->obrazok->vyska = ($sirka_zdroja/$this->sirka)*$this->vyska;
          $this->obrazok->zhora = ($vyska_zdroja - $this->obrazok->vyska)/2;
        }
      } else {
        $temp_vyska = $this->vyska;
        $temp_sirka = $this->sirka;
        if($sirka_zdroja>$vyska_zdroja) {
          $temp_vyska = ($this->sirka/$sirka_zdroja)*$vyska_zdroja;      
        } else {
          $temp_sirka = ($this->vyska/$vyska_zdroja)*$sirka_zdroja;
        }
        if($temp_vyska > $this->vyska) {
          $temp_vyska = $this->vyska;
          $temp_sirka = ($temp_vyska/$vyska_zdroja)*$sirka_zdroja;
        }
        if($temp_sirka > $this->sirka) {
          $temp_sirka = $this->sirka;
          $temp_vyska = ($temp_sirka/$sirka_zdroja)*$vyska_zdroja;
        }
        $this->vyska = $temp_vyska;
        $this->sirka = $temp_sirka;
      }
    }
  }
  
  private function vytvorZoSuboru($cesta,$getimagesizeVal) {
    $akeIm = "";
    switch($getimagesizeVal['mime']) {
      case "image/jpeg":
        $akeIm = imagecreatefromjpeg($cesta);
      break;
      case "image/png":
        $akeIm = imagecreatefrompng($cesta);
      break;
      case "image/gif":
        $akeIm = imagecreatefromgif($cesta);
      break;
    }
    
    if(is_resource($akeIm)) return $akeIm;
    
    return false;
    
  }
  
  private function nastavIm($cesta) {
    
    $this->im = $this->vytvorZoSuboru($cesta,$this->obrazok_info);
    
    if(!is_resource($this->im)) {
      $this->nastavChybu("Nastala chyba pri vytvarani miniatury.");
    }
  }
  
  private function kresliChybu() {
    $fontsize = 2;
    $padding = 10;
    $this->sirka = strlen($this->chyba)*imagefontwidth($fontsize)+$padding*2;
    $this->vyska = imagefontheight($fontsize)+$padding*2;    
    $this->typ = "image/jpeg";
    $this->platno = imagecreatetruecolor($this->sirka, $this->vyska);
    $cierna = imagecolorallocate($this->platno, 0, 0, 0);
    $biela = imagecolorallocate($this->platno, 255, 255, 255);
    imagefilledrectangle($this->platno, 0, 0, $this->sirka, $this->vyska, $cierna);
    imagestring($this->platno, 2, $padding, $padding, $this->chyba, $biela);   
  }
  
  private function kresliMiniaturu() {
    $this->platno = imagecreatetruecolor($this->sirka, $this->vyska);
    
    imagealphablending($this->platno, false);
    imagecopyresampled($this->platno, $this->im, 0, 0, $this->obrazok->zlava, $this->obrazok->zhora, $this->sirka, $this->vyska, $this->obrazok->sirka,$this->obrazok->vyska);
    
    if($this->antiAlias) {
      imageantialias($this->platno, true);
    }
    
    $this->vykonajVsetkyEfekty();
           
  }
  
  private function rotujPlatno() {
    if($this->otoc!=0) {
      $farba_pozadia = imagecolorallocatealpha($this->platno, $this->pozadiePlatna['r'], $this->pozadiePlatna['g'], $this->pozadiePlatna['b'], $this->pozadiePlatna['a']);
      $this->platno = imagerotate($this->platno, $this->otoc, $farba_pozadia);
    }
  }
  
  public function getIm() {
    return $this->im;
  }
  public function getPlatno() {
    return $this->platno;
  }
  
  private function zobrazPlatno($zobrazMiniaturu=true) {
    if(!is_resource($this->platno)) {
      $this->nastavChybu("Chyba.");
      $this->kresliChybu();
      $this->zobrazPlatno();
    }
    if($zobrazMiniaturu) {
      header("Content-type:".$this->typ);
      if($this->cache != 0) header("Cache-Control: max-age=".$this->cache);
    }
    switch($this->typ) {
      case "image/jpeg":
        imagejpeg($this->platno,$this->ulozDo,$this->kvalita);
      break;
      case "image/png":
        imagesavealpha($this->platno, true);
        imagepng($this->platno,$this->ulozDo);
      break;
      case "image/gif":
        imagesavealpha($this->platno, true);
        imagegif($this->platno,$this->ulozDo);
      break;
    }
  }
  
  private function nachystajPlatno() {
    $this->nastavRozmery();
    if($this->chyba) {
      $this->kresliChybu();
    } else {
      $this->kresliMiniaturu();
      if($this->rotujVodoznak) $this->kresliVodoznak();
      $this->rotujPlatno(); 
      if(!$this->rotujVodoznak) $this->kresliVodoznak();
    }
  }
  
  public function ulozDo($cesta,$povolPrepis=false) {
    if(file_exists($cesta) && !$povolPrepis) {
      die("Subor uz existuje!");
    }
    $this->ulozDo = $cesta;
    $this->kresli(false);
  }
  
  public function kresli($zobrazMiniaturu=true) {
    $this->nachystajPlatno();
    $this->zobrazPlatno($zobrazMiniaturu);
  }
  
  
  /*
  * pridávanie Vodoznakov 
  */
  public function pridajVodoznak($cesta="",$x="",$y="",$sirka="",$vyska="") {
    if($cesta=="") return;
    
    $vodoznakData = @getimagesize($cesta);
    if(!isset($vodoznakData[0])) return;
    $sirka = ($sirka=="") ? $vodoznakData[0] : $sirka;
    $vyska = ($vyska=="") ? $vodoznakData[1] : $vyska;
    
    $vodoznakData = array_merge($vodoznakData, array("x"=>$x,"y"=>$y,"sirka"=>$sirka,"vyska"=>$vyska));
    $vodoznakIndex = count($this->vodoznakIm);
    
    $this->vodoznakInfo[$vodoznakIndex] = $vodoznakData;
    $this->vodoznakIm[$vodoznakIndex] = $this->vytvorZoSuboru($cesta,$vodoznakData);
    
    if(!is_resource($this->vodoznakIm[$vodoznakIndex])) {
      $this->nastavChybu("Subor vodoznake nie je spravny.");
    }
  }
  
  private function kresliVodoznak() {
    if(count($this->vodoznakIm)==0) return;
    foreach($this->vodoznakIm as $vI=>$imResource) {
      if(!isset($this->vodoznakInfo[$vI][0])) continue;

      $this->vodoznakXY[$vI] = $this->vypocitajVodoznakXY($this->vodoznakInfo[$vI]);
      imagealphablending($this->platno, true);
      imagecopyresampled($this->platno,
                         $imResource,
                         $this->vodoznakXY[$vI]['x'],
                         $this->vodoznakXY[$vI]['y'],
                         0,
                         0,
                         $this->vodoznakInfo[$vI]['sirka'],
                         $this->vodoznakInfo[$vI]['vyska'],
                         $this->vodoznakInfo[$vI][0],
                         $this->vodoznakInfo[$vI][1]);
    }
  }
  
  private function vypocitajStred($sirka,$sirka_elem) {
    return intval(($sirka/2)-($sirka_elem/2));
  }
  
  private function vypocitajVodoznakXY($data) {
    $vO = $this->vodoznakOdsadenie;
    $x = (imagesx($this->platno) - $data['sirka'] - $vO);
    $y = (imagesy($this->platno) - $data['vyska'] - $vO);
    if(is_integer($data['x'])) $x = $data['x'];
    if(is_integer($data['y'])) $y = $data['y'];
    if((string)$data['x']=="stred") $x = $this->vypocitajStred(imagesx($this->platno),$data['sirka']);
    if((string)$data['y']=="stred") $y = $this->vypocitajStred(imagesy($this->platno),$data['vyska']);
    return array("x" => $x,
                 "y" => $y);
  }
  
  /*
  * vodoznaky - koniec
  */  
  

}

?>